require "core"
require "dbc_file"

# This class assumes WRATH data
# sample on extending dbcfile for re-manipulation of data
class WorldMapArea < DBCFile
  # Fied info
  # 0: ID
  # 1: map id
  # 2: area id
  # 3: Name
  # 4: y1
  # 5: y2
  # 6: x1
  # 7: x2
  # 8: virtualID
  def initialize(path)
      super("#{path}#{File::SEPARATOR}WorldMapArea.dbc","iiisffffIII")
  end
  def get_zone_data()
    zone_info = Hash.new
    #Fields are 0 indexed
    @records.each do |record|
      zone_info[record[3]] = {:ulY => record[4], 
        :lrY => record[5], 
        :ulX => record[6], 
        :lrX => record[7], 
        :width => (record[5].to_f - record[4].to_f).abs,
        :height => (record[7].to_f-record[6].to_f).abs,
        :mapid => record[1],
        :area_id => record[2]
        }
        puts record[3]
    end
    return zone_info
  end
end
class DungeonMap < DBCFile
  #Field info
  #0 subID
  #1 worldmapID
  #2 floor
  #3 uly
  #4 lry
  #5 ulx
  #6 lrx
  #7 i dont know
  def initialize(path)
      super("#{path}#{File::SEPARATOR}DungeonMap.dbc","iiiffffi")
      @map_floors = Hash.new
      @floor_info = Hash.new
      @records.each do |record|
          if @map_floors.has_key?(record[1]) 
            val = @map_floors[record[1]]
            val = val +1
            @map_floors[record[1]] = val
          else
            @map_floors[record[1]] = 1
            @floor_info[record[1]] = Array.new
          end
          @floor_info[record[1]] << {
            :floor => record[2],
            :uly => record[3],
            :lry => record[4],
            :ulx => record[5],
            :lrx => record[6],
            :width => (record[4].to_f - record[3].to_f).abs,
            :height => (record[6].to_f-record[5].to_f).abs
          }
      end
  end
  def get_floors(mapID)
    if @map_floors.has_key?(mapID) and mapID > 500
      return @map_floors[mapID]
    else
      return 0
    end
  end
  def get_floor_data(mapID)
    return @floor_info[mapID]
  end
end
# Cmdline Usage
if ARGV.nil? 
  puts "Usage: #{$0} <path to dbc files>"
  exit(0)
end
path = ARGV[0]
unless File.directory?(path)
  puts "Usage: #{$0} <path to dbc files>"
  exit(0)  
end

zoneMap = Hash.new
zoneMap["Arathi"] = 1
zoneMap["Ogrimmar"] = 2
zoneMap["EasternKingdoms"] =3
zoneMap["Undercity"] = 4
zoneMap["Barrens"] =5
zoneMap["Darnassis"] =6
zoneMap["AzuremystIsle"] =7
zoneMap["UngoroCrater"] =8
zoneMap["BurningSteppes"] =9
zoneMap["Wetlands"] =10
zoneMap["Winterspring"] =11
zoneMap["Dustwallow"] =12
zoneMap["Darkshore"] =13
zoneMap["LochModan"] =14
zoneMap["BladesEdgeMountains"] =15
zoneMap["Durotar"] =16
zoneMap["Silithus"] =17
zoneMap["ShattrathCity"] =18
zoneMap["Ashenvale"] =19
zoneMap["Azeroth"] =20
zoneMap["Nagrand"] =21
zoneMap["TerokkarForest"] =22
zoneMap["EversongWoods"] =23
zoneMap["SilvermoonCity"] =24
zoneMap["Tanaris"] =25
zoneMap["Stormwind"] =26
zoneMap["SwampOfSorrows"] =27
zoneMap["EasternPlaguelands"] =28
zoneMap["BlastedLands"] =29
zoneMap["Elwynn"] =30
zoneMap["DeadwindPass"] =31
zoneMap["DunMorogh"] =32
zoneMap["TheExodar"] =33
zoneMap["Felwood"] =34
zoneMap["Silverpine"] =35
zoneMap["ThunderBluff"] =36
zoneMap["Hinterlands"] =37
zoneMap["StonetalonMountains"] =38
zoneMap["Mulgore"] =39
zoneMap["Hellfire"] =40
zoneMap["Ironforge"] =41
zoneMap["ThousandNeedles"] =42
zoneMap["Stranglethorn"] =43
zoneMap["Badlands"] =44
zoneMap["Teldrassil"] =45
zoneMap["Moonglade"] =46
zoneMap["ShadowmoonValley"] =47
zoneMap["Tirisfal"] =48
zoneMap["Aszhara"] =49
zoneMap["Redridge"] =50
zoneMap["BloodmystIsle"] =51
zoneMap["WesternPlaguelands"] =52
zoneMap["Alterac"] =53
zoneMap["Westfall"] =54
zoneMap["Duskwood"] =55
zoneMap["Netherstorm"] =56
zoneMap["Ghostlands"] =57
zoneMap["Zangarmarsh"] =58
zoneMap["Desolace"] =59
zoneMap["Kalimdor"] =60
zoneMap["SearingGorge"] =61
zoneMap["Expansion01"] =62
zoneMap["Feralas"] =63
zoneMap["Hilsbrad"] =64
zoneMap["Sunwell"] =65
zoneMap["Northrend"] =66
zoneMap["BoreanTundra"] =67
zoneMap["Dragonblight"] =68
zoneMap["GrizzlyHills"] =69
zoneMap["HowlingFjord"] =70
zoneMap["IcecrownGlacier"] =71
zoneMap["SholazarBasin"] =72
zoneMap["TheStormPeaks"] =73
zoneMap["ZulDrak"] =74
zoneMap["ScarletEnclave"] =76
zoneMap["CrystalsongForest"] =77
zoneMap["LakeWintergrasp"] =78
zoneMap["StrandoftheAncients"] =79
zoneMap["Dalaran"] =80

zoneData = WorldMapArea.new("#{path}") 
floorMap = DungeonMap.new("#{path}")

zoneMaps = zoneData.get_zone_data
max_zone= 81
final_data = Array.new
zoneMaps.each_pair do |map,data|
  zoneNumber = max_zone
  if zoneMap.has_key?(map)
    zoneNumber = zoneMap[map]
  else
    zoneNumber = max_zone
    max_zone = max_zone + 1
  end
  final_data[zoneNumber] = "#{map} = { #{data[:width]}, #{data[:height]}, #{zoneNumber}},"
  #puts "#{map} = { #{data[:height]}, #{data[:width]}, #{zoneNumber}},"
  floors = floorMap.get_floors(data[:mapid])
  if floors > 0 
      if data[:mapid] == 571
        if data[:area_id] == 4395
          fdata = floorMap.get_floor_data(data[:mapid])
        end
      else
        fdata = floorMap.get_floor_data(data[:mapid])
      end
      if !fdata.nil?
        fdata.each do |floor|
          zoneNumber = max_zone
          max_zone = max_zone + 1
          final_data[zoneNumber] = "#{map}#{floor[:floor]} = { #{floor[:width]}, #{floor[:height]}, #{zoneNumber}},"
        end
      end
  end
end

final_data.each do |rcd|
  if rcd then
    puts rcd
  end
end

#instance = "Ulduar"
#map = zoneData.get_zone_data[instance][:mapid]
#puts zoneData.get_zone_data[instance].inspect
#puts "Floors of #{instance} #{floorMap.get_floors(map)}"

#floors = floorMap.get_floor_data(map)
#if floors
#  floors.each do |floor|
#    puts "Floor #{floor[:floor]} Height: #{floor[:height]} Widht: #{floor[:width]}"
#  end
#end
#puts zoneData.get_zone_data.to_lua